/* === This file is part of Calamares - <https://calamares.io> ===
 *
 *   SPDX-FileCopyrightText: 2014 Teo Mrnjavac <teo@kde.org>
 *   SPDX-License-Identifier: GPL-3.0-or-later
 *
 *   Calamares is Free Software: see the License-Identifier above.
 *
 */

#ifndef PARTMAN_DEVICES_H
#define PARTMAN_DEVICES_H

#ifdef __cplusplus
extern "C"
{
#endif

    int check_big_enough( long long required_space );

#ifdef __cplusplus
}  // extern "C"
#endif

#endif  // PARTMAN_DEVICES_H
