
#ifndef CALAMARES_TRANSLATIONS_H
#define CALAMARES_TRANSLATIONS_H
#include <QStringList>
namespace {
static const QStringList availableLanguageList{
"ast",
"bg",
"ca",
"cs",
"da",
"de",
"en",
"es",
"es_AR",
"fi",
"fr",
"hi_IN",
"hr",
"hu",
"id_ID",
"it",
"ja",
"nl_NL",
"pl",
"pt_BR",
"pt_PT",
"ro_RO",
"ru",
"sl",
"sr_RS",
"sv",
"tr_TR",
};
} // namespace
#endif

